@@ -422,11 +422,12 @@ anjuta_launcher_send_stdin_eof (AnjutaLauncher *launcher)
 void
 anjuta_launcher_send_ptyin (AnjutaLauncher *launcher, const gchar * input_str)
 {
-	gsize bytes_written;
+	gsize bytes_written = 0;
 	GError *err = NULL;
 	
 	g_return_if_fail (launcher);
 	g_return_if_fail (input_str);
+	g_return_if_fail (launcher->priv->pty_channel != NULL);
 	
 	if (strlen (input_str) == 0) 
 		return;
@@ -460,7 +461,8 @@ anjuta_launcher_send_ptyin (AnjutaLauncher *launcher, const gchar * input_str)
 void
 anjuta_launcher_reset (AnjutaLauncher *launcher)
 {
-	if (anjuta_launcher_is_busy (launcher))
+	if (anjuta_launcher_is_busy (launcher) &&
+	    launcher->priv->child_pid)
 		kill (launcher->priv->child_pid, SIGTERM);
 }
 
@@ -474,7 +476,8 @@ anjuta_launcher_reset (AnjutaLauncher *launcher)
 void
 anjuta_launcher_signal (AnjutaLauncher *launcher, int sig)
 {
-	kill (launcher->priv->child_pid, sig);
+	if (launcher->priv->child_pid)
+		kill (launcher->priv->child_pid, sig);
 }
 
 /**
