@@ -647,6 +647,14 @@ anjuta_token_remove_word (AnjutaToken *token)
 	return next;
 }
 
+/**
+ * anjuta_token_remove_list:
+ * @token: a #AnjutaToken a list element
+ *
+ * Remove a complete list of token.
+ *
+ * Return value: A #AnjutaToken representing the following token
+ */
 AnjutaToken *
 anjuta_token_remove_list (AnjutaToken *token)
 {
@@ -658,41 +666,49 @@ anjuta_token_remove_list (AnjutaToken *token)
 	anjuta_token_set_flags (list, ANJUTA_TOKEN_REMOVED);
 
 	prev = anjuta_token_previous_item (list);
-	if (anjuta_token_get_type (prev) == ANJUTA_TOKEN_EOL)
+	if (prev != NULL)
 	{
-		/* Remove line above if empty */
-		if (anjuta_token_get_type (anjuta_token_previous_item (prev)) == ANJUTA_TOKEN_EOL)
+		if (anjuta_token_get_type (prev) == ANJUTA_TOKEN_EOL)
 		{
-			anjuta_token_set_flags (prev, ANJUTA_TOKEN_REMOVED);
-		}
-	}
-	else if (anjuta_token_get_type (prev) == ANJUTA_TOKEN_COMMENT)
-	{
-		/* Remove comment above if there is an empty line after it */
-		do
-		{
-			prev = anjuta_token_previous_item (prev);
-		}
-		while ((prev != NULL) && (anjuta_token_get_type (prev) == ANJUTA_TOKEN_COMMENT));
-
-		if ((prev != NULL) && (anjuta_token_get_type (prev) == ANJUTA_TOKEN_EOL))
-		{
-			prev = list;
-			do
+			/* Remove line above if empty */
+			AnjutaToken *prev_prev = anjuta_token_previous_item (prev);
+		
+			if ((prev_prev == NULL) || (anjuta_token_get_type (prev_prev) == ANJUTA_TOKEN_EOL) || (anjuta_token_get_type (prev_prev) == ANJUTA_TOKEN_COMMENT))
 			{
 				anjuta_token_set_flags (prev, ANJUTA_TOKEN_REMOVED);
+			}
+		}
+		else if (anjuta_token_get_type (prev) == ANJUTA_TOKEN_COMMENT)
+		{
+			/* Remove comment above if there is an empty line after it */
+			do
+			{
 				prev = anjuta_token_previous_item (prev);
 			}
 			while ((prev != NULL) && (anjuta_token_get_type (prev) == ANJUTA_TOKEN_COMMENT));
+
+			if ((prev != NULL) && (anjuta_token_get_type (prev) == ANJUTA_TOKEN_EOL))
+			{
+				prev = list;
+				do
+				{
+					anjuta_token_set_flags (prev, ANJUTA_TOKEN_REMOVED);
+					prev = anjuta_token_previous_item (prev);
+				}
+				while ((prev != NULL) && (anjuta_token_get_type (prev) == ANJUTA_TOKEN_COMMENT));
+			}
 		}
 	}
 	
 	next = anjuta_token_next_item (list);
-	if (anjuta_token_get_type (next) == ANJUTA_TOKEN_EOL)
+	if (next != NULL)
 	{
-		anjuta_token_set_flags (next, ANJUTA_TOKEN_REMOVED);
+		if (anjuta_token_get_type (next) == ANJUTA_TOKEN_EOL)
+		{
+			anjuta_token_set_flags (next, ANJUTA_TOKEN_REMOVED);
+		}
+		next = anjuta_token_next_item (next);
 	}
-	next = anjuta_token_next_item (next);
 	
 	return next;
 }
