@@ -64,7 +64,7 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 		g_return_val_if_fail (pm != NULL, FALSE);
 		exec_targets =
 		ianjuta_project_manager_get_targets (pm, 
-							ANJUTA_TARGET_EXECUTABLE,
+							ANJUTA_PROJECT_TARGET | ANJUTA_PROJECT_EXECUTABLE,
                             NULL);
 		if (!exec_targets)
 		{
@@ -112,12 +112,13 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 		while (node)
 		{
 			gchar *local_path;
+            gchar *uri;
 			const gchar *rel_path;
-		
-			local_path = anjuta_util_get_local_path_from_uri ((gchar *)node->data);
+
+			local_path = g_file_get_path ((GFile *)node->data);
 			if (local_path == NULL)
 			{
-				g_free (node->data);
+				g_object_unref (node->data);
 				node = g_list_next (node);
 				continue;
 			}
@@ -126,11 +127,12 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 				(gchar*)local_path +
 				 strlen (plugin->project_root_dir) + 1;
 			
+            uri = g_file_get_uri ((GFile *)node->data);
 			gtk_list_store_append (store, &iter);
 			gtk_list_store_set (store, &iter, 0, rel_path, 1,
-								node->data, -1);
+								uri, -1);
       
-      if (plugin->last_exec_uri && g_str_equal (plugin->last_exec_uri, node->data))
+      if (plugin->last_exec_uri && g_str_equal (plugin->last_exec_uri, uri))
       { 
 				GtkTreePath* path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
         gtk_tree_selection_select_iter (selection, &iter);
@@ -141,9 +143,10 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
         g_free (plugin->last_exec_uri);
         plugin->last_exec_uri = NULL;
       }
-      
+
+            g_free (uri);
 			g_free (local_path);
-			g_free (node->data);
+			g_object_unref (node->data);
 			node = g_list_next (node);
 		}
 		g_list_free (exec_targets);
@@ -219,7 +222,7 @@ get_program_parameters (BasicAutotoolsPlugin *plugin,
 void
 execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 {
-	AnjutaPreferences* prefs = anjuta_shell_get_preferences (ANJUTA_PLUGIN(plugin)->shell, NULL);
+	GSettings* settings = plugin->settings;
 	gboolean run_in_terminal, error_condition;
 	gchar *target = NULL, *args = NULL, *dir = NULL, *cmd = NULL;
 
@@ -372,9 +375,9 @@ execute_program (BasicAutotoolsPlugin* plugin, const gchar *pre_select_uri)
 	else
 		cmd = g_strdup (target);
 
-	if (anjuta_preferences_get_bool (prefs , PREF_USE_SB))
+	if (g_settings_get_boolean (settings, PREF_USE_SB))
 	{
-		const gchar* sb_path = anjuta_preferences_get(prefs, PREF_SB_PATH);
+		const gchar* sb_path = g_settings_get_string (settings, PREF_SB_PATH);
 		/* we need to skip the /scratchbox/users part, maybe could be done more clever */
 		const gchar* real_dir = strstr(target, "/home");
 		gchar* oldcmd = cmd;
