@@ -162,8 +162,8 @@ static GtkActionEntry actions_file[] = {
   { "ActionFileCloseAll", GTK_STOCK_CLOSE, N_("Close All"), "<shift><control>w",
 	N_("Close all files"),
     G_CALLBACK (on_close_all_file_activate)},
-  { "ActionFileCloseOther", GTK_STOCK_CLOSE, N_("Close Other"), "<shift><control>o",
-	N_("Close other documents"),
+  { "ActionFileCloseOther", GTK_STOCK_CLOSE, N_("Close Others"), "<shift><control>o",
+	N_("Close others documents"),
     G_CALLBACK (on_close_other_file_activate)},	
   { "ActionFileReload", GTK_STOCK_REVERT_TO_SAVED, N_("Reload F_ile"), NULL,
 	N_("Reload current file"),
@@ -266,11 +266,33 @@ static GtkActionEntry actions_search[] = {
   { "ActionEditSearchQuickSearch", GTK_STOCK_FIND, N_("_Quick Search"),
 	"<control>f", N_("Quick editor embedded search"),
     G_CALLBACK (on_show_search)},
-  { "ActionEditSearchQuickSearchAgain", GTK_STOCK_FIND, N_("Quick _ReSearch"),
-	"<control><shift>f", N_("Repeat quick search"),
-    G_CALLBACK (on_repeat_quicksearch)}
+  { "ActionEditSearchQuickSearchAgain", GTK_STOCK_FIND, N_("Find _Next"),
+	"<control>g", N_("Search for next appearance of term."),
+    G_CALLBACK (on_repeat_quicksearch)},
+  { "ActionEditSearchReplace", GTK_STOCK_FIND, N_("Find and R_eplace…"),
+	"<control>h", N_("Search and replace"),
+    G_CALLBACK (on_search_and_replace)},
+  { "ActionEditSearchFindPrevious", GTK_STOCK_FIND, N_("Find _Previous"),
+	"<control><shift>g",
+	N_("Repeat the last Find command"),
+	G_CALLBACK (on_search_previous)},	
+  { "ActionSearchboxPopupClearHighlight", GTK_STOCK_FIND, N_("Clear Highlight"),
+	NULL, N_("Clear all highlighted text"),
+	G_CALLBACK (on_search_popup_clear_highlight)}
 };
 
+static GtkToggleActionEntry actions_searchbox_popup[] = {
+  { "ActionSearchboxPopupCaseCheck", GTK_STOCK_FIND, 
+    N_("Case Sensitive"), NULL, 
+    N_("Match case in search results."),
+	G_CALLBACK (on_search_popup_case_sensitive_toggle)},
+  { "ActionSearchboxPopupHighlightAll", GTK_STOCK_FIND, 
+    N_("Highlight All"), NULL, 
+    N_("Highlight all occurrences"),
+	G_CALLBACK (on_search_popup_highlight_toggle)}
+};
+
+
 static GtkActionEntry actions_edit[] = {
   { "ActionMenuEdit", NULL, N_("_Edit"), NULL, NULL, NULL},/* menu title */
   { "ActionMenuViewEditor", NULL, N_("_Editor"), NULL, NULL, NULL},
@@ -316,6 +338,7 @@ static GtkActionEntry actions_edit[] = {
 	}
 };
 
+
 static GtkToggleActionEntry actions_view[] = {
   { "ActionViewEditorLinenumbers", NULL, N_("_Line Number Margin"), NULL,
 	N_("Show/Hide line numbers"),
@@ -368,10 +391,10 @@ static GtkActionEntry actions_format[] = {
 static GtkActionEntry actions_documents[] = {
   {"ActionMenuDocuments", NULL, N_("_Documents"), NULL, NULL, NULL},
   { "ActionDocumentsPrevious", GTK_STOCK_GO_BACK, N_("Previous Document"),
-	"<control><alt>Page_Up", N_("Switch to previous document"),
+	"<control>Page_Up", N_("Switch to previous document"),
     G_CALLBACK (on_previous_document)},
   { "ActionDocumentsNext", GTK_STOCK_GO_FORWARD, N_("Next Document"),
-	"<control><alt>Page_Down", N_("Switch to next document"),
+	"<control>Page_Down", N_("Switch to next document"),
     G_CALLBACK (on_next_document)},
 };
 
@@ -423,7 +446,9 @@ static struct ActionGroupInfo action_groups[] = {
 };
 
 static struct ActionToggleGroupInfo action_toggle_groups[] = {
-	{ actions_view, G_N_ELEMENTS (actions_view), "ActionGroupEditorView", N_("Editor view settings") }
+	{ actions_view, G_N_ELEMENTS (actions_view), "ActionGroupEditorView", N_("Editor view settings") },
+	{ actions_searchbox_popup, G_N_ELEMENTS(actions_searchbox_popup), 
+	  "ActionGroupEditorSearchOptions", N_("Toggle search options")	}
 };
 
 // void pref_set_style_combo(DocmanPlugin *plugin);
@@ -845,10 +870,24 @@ update_document_ui_interface_items (AnjutaPlugin *plugin, IAnjutaDocument *doc)
 	g_object_set (G_OBJECT (action), "sensitive", flag, NULL);
 	action = anjuta_ui_get_action (ui,  "ActionGroupEditorSearch",
 								   "ActionEditSearchQuickSearchAgain");
+	action = anjuta_ui_get_action (ui,  "ActionGroupEditorSearch",
+								   "ActionEditSearchFindPrevious");
+	g_object_set (G_OBJECT (action), "sensitive", flag, NULL);
+	action = anjuta_ui_get_action (ui, "ActionGroupEditorSearch",
+									"ActionEditSearchReplace");
+	g_object_set (G_OBJECT (action), "sensitive", flag, NULL);
+	action = anjuta_ui_get_action (ui,  "ActionGroupEditorSearch",
+								   "ActionSearchboxPopupClearHighlight");
 	g_object_set (G_OBJECT (action), "sensitive", flag, NULL);
 	action = anjuta_ui_get_action (ui,  "ActionGroupEditorNavigate",
 								   "ActionEditGotoLine");	
 	g_object_set (G_OBJECT (action), "sensitive", flag, NULL);
+	action = anjuta_ui_get_action (ui,  "ActionGroupEditorSearchOptions",
+								   "ActionSearchboxPopupCaseCheck");
+	g_object_set (G_OBJECT (action), "sensitive", flag, NULL);
+	action = anjuta_ui_get_action (ui,  "ActionGroupEditorSearchOptions",
+								   "ActionSearchboxPopupHighlightAll");
+	g_object_set (G_OBJECT (action), "sensitive", flag, NULL);
 
 	/* IAnjutaEditorAssist */
 	flag = IANJUTA_IS_EDITOR_ASSIST (doc);
@@ -1107,10 +1146,12 @@ unload_unused_support_plugins (DocmanPlugin* docman_plugin,
 {
 	GList* plugins = g_list_copy (docman_plugin->support_plugins);
 	GList* node;
+	DEBUG_PRINT ("Unloading plugins");
 	for (node = plugins; node != NULL; node = g_list_next (node))
 	{
 		AnjutaPlugin* plugin = ANJUTA_PLUGIN (node->data);
-		if (!g_list_find (needed_plugins, plugin))
+		DEBUG_PRINT ("Checking plugin: %p", plugin);
+		if (g_list_find (needed_plugins, plugin) == NULL)
 		{
 			DEBUG_PRINT ("%s", "Unloading plugin");
 			anjuta_plugin_deactivate (plugin);
@@ -1123,16 +1164,15 @@ static void
 update_language_plugin (AnjutaDocman *docman, IAnjutaDocument *doc,
                         AnjutaPlugin *plugin)
 {
-	if (IANJUTA_IS_EDITOR_LANGUAGE (doc))
+	DocmanPlugin* docman_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
+
+	DEBUG_PRINT("%s", "Beginning language support");
+	if (doc && IANJUTA_IS_EDITOR_LANGUAGE (doc))
 	{
-		DEBUG_PRINT("%s", "Beginning language support");
 		AnjutaPluginManager *plugin_manager;
-		DocmanPlugin* docman_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 		IAnjutaLanguage *lang_manager;
 		GList *new_support_plugins, *support_plugin_descs, *needed_plugins, *node;
 		const gchar *language;
-
-		g_signal_handlers_block_by_func (doc, on_editor_lang_changed, plugin);
 		
 		lang_manager = anjuta_shell_get_interface (plugin->shell,
 		                                           IAnjutaLanguage,
@@ -1142,18 +1182,16 @@ update_language_plugin (AnjutaDocman *docman, IAnjutaDocument *doc,
 			g_warning ("Could not load language manager!");
 			return;
 		}
+
+		g_signal_handlers_block_by_func (doc, on_editor_lang_changed, plugin);
 		language = ianjuta_language_get_name_from_editor (lang_manager,
 		                                                  IANJUTA_EDITOR_LANGUAGE (doc),
 		                                                  NULL);
+		g_signal_handlers_unblock_by_func (doc, on_editor_lang_changed, plugin);
+		
 		if (!language)
 		{
-			DEBUG_PRINT ("%s", "Unloading all language support plugins");
-			/* Unload all language support plugins */
-			/* Copy the list because the "deactivate"-signal handler modifies
-			 * the original list */
-			GList* plugins = g_list_copy (docman_plugin->support_plugins);
-			g_list_foreach (plugins, (GFunc) anjuta_plugin_deactivate, NULL);
-			g_list_free (plugins);
+			unload_unused_support_plugins (docman_plugin, NULL);
 			return;
 		}
 
@@ -1192,14 +1230,12 @@ update_language_plugin (AnjutaDocman *docman, IAnjutaDocument *doc,
 		g_list_free (docman_plugin->support_plugins);
 		docman_plugin->support_plugins = needed_plugins;
 
-		if (new_support_plugins)
-		{
-			g_list_foreach (new_support_plugins, (GFunc) g_free, NULL);
-			g_list_free (new_support_plugins);
-		}
-
-		g_signal_handlers_unblock_by_func (doc, on_editor_lang_changed, plugin);
-	}				 
+		anjuta_util_glist_strings_free (new_support_plugins);
+	}
+	else
+	{
+		unload_unused_support_plugins (docman_plugin, NULL);
+	}
 }
 
 static void
@@ -1233,8 +1269,10 @@ on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 		update_language_plugin (docman, doc, plugin);
 	}
 	else
+	{
 		update_status (docman_plugin, NULL);
-
+		update_language_plugin (docman, NULL, plugin);
+	}
 	update_title (ANJUTA_PLUGIN_DOCMAN (plugin));
 }
 
@@ -1766,9 +1804,9 @@ activate_plugin (AnjutaPlugin *plugin)
 											"/PopupDocumentManager");
 	g_assert (popup_menu != NULL && GTK_IS_MENU (popup_menu));
 	anjuta_docman_set_popup_menu (ANJUTA_DOCMAN (docman), popup_menu);
+
 	if (!initialized)
 	{
-		//search_and_replace_init (ANJUTA_DOCMAN (docman));
 	}
 	/* Connect to save session */
 	g_signal_connect (G_OBJECT (plugin->shell), "save-session",
@@ -2177,7 +2215,7 @@ ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError**
 static void
 ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
-	DocmanPlugin* plugin = ANJUTA_PLUGIN_DOCMAN (ipref);
+	/*	DocmanPlugin* plugin = ANJUTA_PLUGIN_DOCMAN (ipref);*/
 	anjuta_preferences_remove_page(prefs, _("Documents"));
 }
 
