@@ -24,7 +24,7 @@
 
 #include "git-status-command.h"
 
-#define STATUS_REGEX "(modified|new file|deleted|unmerged):   (.*)"
+#define STATUS_REGEX "(modified|new file|deleted|unmerged|both modified|both added|both deleted):   (.*)"
 #define UNTRACKED_FILES_REGEX "(?:#\\t)(.*)"
 #define SECTION_COMMIT_REGEX "Changes to be committed:"
 #define SECTION_NOT_UPDATED_REGEX "Changed but not updated:"
@@ -42,6 +42,8 @@ struct _GitStatusCommandPriv
 	GRegex *section_commit_regex;
 	GRegex *section_not_updated_regex;
 	GRegex *section_untracked_regex;
+	GFileMonitor *head_monitor;
+	GFileMonitor *index_monitor;
 };
 
 G_DEFINE_TYPE (GitStatusCommand, git_status_command, GIT_TYPE_COMMAND);
@@ -145,6 +147,125 @@ git_status_command_init (GitStatusCommand *self)
 													   0, 0, NULL);
 }
 
+static void
+on_file_monitor_changed (GFileMonitor *monitor, GFile *file, GFile *other_file,
+                         GFileMonitorEvent event, AnjutaCommand *command)
+{
+	/* Handle created and modified events just to cover all possible cases. 
+	 * Sometimes git does some odd things... */
+	if (event == G_FILE_MONITOR_EVENT_CHANGED ||
+	    event == G_FILE_MONITOR_EVENT_CREATED)
+	{
+		anjuta_command_start (command);
+	}
+}
+
+static gboolean
+git_status_command_start_automatic_monitor (AnjutaCommand *command)
+{
+	GitStatusCommand *self;
+	gchar *working_directory;
+	gchar *git_head_path;
+	gchar *git_index_path;
+	GFile *git_head_file;
+	GFile *git_index_file;
+
+	self = GIT_STATUS_COMMAND (command);
+
+	g_object_get (self, "working-directory", &working_directory, NULL);
+
+	/* Watch for changes to the HEAD file and the index file, so that we can
+	 * at least detect commits and index changes. */
+	git_head_path = g_strjoin (G_DIR_SEPARATOR_S,
+	                           working_directory,
+	                           ".git",
+	                           "HEAD",
+	                           NULL);
+	git_index_path = g_strjoin (G_DIR_SEPARATOR_S,
+	                            working_directory,
+	                            ".git",
+	                            "index",
+	                            NULL);
+	git_head_file = g_file_new_for_path (git_head_path);
+	git_index_file = g_file_new_for_path (git_index_path);
+	self->priv->head_monitor = g_file_monitor_file (git_head_file, 0, NULL, 
+	                                                NULL);
+	self->priv->index_monitor = g_file_monitor_file (git_index_file, 0, NULL,
+	                                                 NULL);
+
+	g_signal_connect (G_OBJECT (self->priv->head_monitor), "changed",
+	                  G_CALLBACK (on_file_monitor_changed),
+	                  command);
+
+	g_signal_connect (G_OBJECT (self->priv->index_monitor), "changed",
+	                  G_CALLBACK (on_file_monitor_changed),
+	                  command);
+
+	g_free (git_head_path);
+	g_free (git_index_path);
+	g_object_unref (git_head_file);
+	g_object_unref (git_index_file);
+
+	return TRUE;
+}
+
+static void
+git_status_command_stop_automatic_monitor (AnjutaCommand *command)
+{
+	GitStatusCommand *self;
+
+	self = GIT_STATUS_COMMAND (command); 
+
+	if (self->priv->head_monitor)
+	{
+		g_file_monitor_cancel (self->priv->head_monitor);
+		g_object_unref (self->priv->head_monitor);
+		self->priv->head_monitor = NULL;
+	}
+
+	if (self->priv->index_monitor)
+	{
+		g_file_monitor_cancel (self->priv->index_monitor);
+		g_object_unref (self->priv->index_monitor);
+		self->priv->index_monitor = NULL;
+	}
+}
+
+static void
+git_status_command_clear_output (GitStatusCommand *self)
+{
+	GList *current_output;
+
+	current_output = self->priv->status_queue->head;
+
+	while (current_output)
+	{
+		g_object_unref (current_output->data);
+		current_output = g_list_next (current_output);
+	}
+
+	g_queue_clear (self->priv->status_queue);
+}
+
+static void
+git_status_command_data_arrived (AnjutaCommand *command)
+{
+	git_status_command_clear_output (GIT_STATUS_COMMAND (command));
+}
+
+static void
+git_status_command_finished (AnjutaCommand *command, guint return_code)
+{
+	GitStatusCommand *self;
+
+	self = GIT_STATUS_COMMAND (command);
+
+	g_hash_table_remove_all (self->priv->path_lookup_table);
+
+	ANJUTA_COMMAND_CLASS (git_status_command_parent_class)->command_finished (command, 
+	                                                                          return_code);
+}
+
 static void
 git_status_command_finalize (GObject *object)
 {
@@ -154,11 +275,8 @@ git_status_command_finalize (GObject *object)
 	self = GIT_STATUS_COMMAND (object);
 	current_status = self->priv->status_queue->head;
 	
-	while (current_status)
-	{
-		g_object_unref (current_status->data);
-		current_status = g_list_next (current_status);
-	}
+	git_status_command_clear_output (self);
+	git_status_command_stop_automatic_monitor (ANJUTA_COMMAND (self));
 	
 	g_queue_free (self->priv->status_queue);
 	g_hash_table_destroy (self->priv->path_lookup_table);
@@ -183,6 +301,10 @@ git_status_command_class_init (GitStatusCommandClass *klass)
 	object_class->finalize = git_status_command_finalize;
 	parent_class->output_handler = git_status_command_handle_output;
 	command_class->run = git_status_command_run;
+	command_class->data_arrived = git_status_command_data_arrived;
+	command_class->command_finished = git_status_command_finished;
+	command_class->start_automatic_monitor = git_status_command_start_automatic_monitor;
+	command_class->stop_automatic_monitor = git_status_command_stop_automatic_monitor;
 }
 
 
