@@ -29,6 +29,7 @@
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-message-manager.xml"
 #define PREFS_BUILDER PACKAGE_DATA_DIR"/glade/anjuta-message-manager-plugin.ui"
+#define PREFERENCES_SCHEMA "org.gnome.anjuta.message-manager"
 
 /* Pixmaps */
 #define ANJUTA_PIXMAP_MESSAGES                "anjuta-messages-plugin-48.png"
@@ -92,7 +93,8 @@ static GtkActionEntry actions_goto[] = {
     G_CALLBACK (on_save_message)}
 };
 
-static void on_view_changed(AnjutaMsgman* msgman, MessageViewPlugin* plugin)
+static void on_view_changed(AnjutaMsgman* msgman, GtkWidget* page,
+                            gint page_num, MessageViewPlugin* plugin)
 {
 	MessageView* view = anjuta_msgman_get_current_view (msgman);
 	AnjutaUI* ui = anjuta_shell_get_ui (ANJUTA_PLUGIN(plugin)->shell, NULL);
@@ -146,67 +148,6 @@ register_stock_icons (AnjutaPlugin *plugin)
 	END_REGISTER_ICON;
 }
 
-#if 0 /* Disable session saving/loading until a way is found to avoid
-       * number of message panes infinitely growing */
-static void
-on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
-				 AnjutaSession *session, MessageViewPlugin *plugin)
-{
-	gboolean success;
-	const gchar *dir;
-	gchar *messages_file;
-	AnjutaSerializer *serializer;
-		
-	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
-		return;
-	
-	dir = anjuta_session_get_session_directory (session);
-	messages_file = g_build_filename (dir, "messages.txt", NULL);
-	serializer = anjuta_serializer_new (messages_file,
-										ANJUTA_SERIALIZER_WRITE);
-	if (!serializer)
-	{
-		g_free (messages_file);
-		return;
-	}
-	success = anjuta_msgman_serialize (ANJUTA_MSGMAN (plugin->msgman),
-									   serializer);
-	g_object_unref (serializer);
-	if (!success)
-	{
-		g_warning ("Serialization failed: deleting %s", messages_file);
-		unlink (messages_file);
-	}
-	g_free (messages_file);
-}
-
-static void
-on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
-				 AnjutaSession *session, MessageViewPlugin *plugin)
-{
-	const gchar *dir;
-	gchar *messages_file;
-	AnjutaSerializer *serializer;
-	
-	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
-		return;
-	
-	dir = anjuta_session_get_session_directory (session);
-	messages_file = g_build_filename (dir, "messages.txt", NULL);
-	serializer = anjuta_serializer_new (messages_file,
-										ANJUTA_SERIALIZER_READ);
-	if (!serializer)
-	{
-		g_free (messages_file);
-		return;
-	}
-	anjuta_msgman_remove_all_views (ANJUTA_MSGMAN (plugin->msgman));
-	anjuta_msgman_deserialize (ANJUTA_MSGMAN (plugin->msgman), serializer);
-	g_object_unref (serializer);
-	g_free (messages_file);
-}
-#endif
-
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
@@ -233,7 +174,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (ui), "/PopupMessageView");
 	mv_plugin->msgman = 
 		anjuta_msgman_new(popup);
-	g_signal_connect(G_OBJECT(mv_plugin->msgman), "view_changed", 
+	g_signal_connect(mv_plugin->msgman, "switch-page", 
 					 G_CALLBACK(on_view_changed), mv_plugin);
 	GtkAction* action_next = anjuta_ui_get_action (ui, "ActionGroupGotoMessages",
 								   "ActionMessageNext");
@@ -245,13 +186,6 @@ activate_plugin (AnjutaPlugin *plugin)
 	g_object_set (G_OBJECT (action_prev), "sensitive", FALSE, NULL);
 	g_object_set (G_OBJECT (action_copy), "sensitive", FALSE, NULL);
 	
-#if 0
-	/* Connect to save and load session */
-	g_signal_connect (G_OBJECT (plugin->shell), "save-session",
-					  G_CALLBACK (on_session_save), plugin);
-	g_signal_connect (G_OBJECT (plugin->shell), "load-session",
-					  G_CALLBACK (on_session_load), plugin);
-#endif
 	initialized = TRUE;
 	mv_plugin->widget_shown = FALSE;
 	return TRUE;
@@ -266,15 +200,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	DEBUG_PRINT ("%s", "MessageViewPlugin: Dectivating message view plugin ...");
 	
 	mplugin = ANJUTA_PLUGIN_MESSAGE_VIEW (plugin);
-#if 0
-	/* Disconnect signals */
-	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
-										  G_CALLBACK (on_session_save),
-										  plugin);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
-										  G_CALLBACK (on_session_load),
-										  plugin);
-#endif
+
 	/* Widget is removed as soon as it is destroyed */
 	if (mplugin->widget_shown)
 		gtk_widget_destroy (mplugin->msgman);
@@ -291,7 +217,8 @@ deactivate_plugin (AnjutaPlugin *plugin)
 static void
 message_view_plugin_dispose (GObject *obj)
 {
-	// MessageViewPlugin *plugin = ANJUTA_PLUGIN_MESSAGE_VIEW (obj);
+	MessageViewPlugin *plugin = ANJUTA_PLUGIN_MESSAGE_VIEW (obj);
+	g_object_unref (plugin->settings);
 	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
@@ -309,6 +236,7 @@ message_view_plugin_instance_init (GObject *obj)
 	plugin->action_group = NULL;
 	plugin->msgman = NULL;
 	plugin->uiid = 0;
+	plugin->settings = g_settings_new (PREFERENCES_SCHEMA);
 }
 
 static void
@@ -375,6 +303,8 @@ on_filter_button_tooltip (GtkWidget* widget,
 	else
 		g_assert_not_reached ();
 
+	g_free (temp);
+
 	return TRUE;
 }
 
@@ -569,8 +499,6 @@ ianjuta_msgman_iface_init (IAnjutaMessageManagerIface *iface)
 	iface->set_view_icon_from_stock = ianjuta_msgman_set_view_icon_from_stock;
 }
 
-static guint notify_id;
-
 static void
 ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
@@ -583,20 +511,20 @@ ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError**
 	{
 		g_warning ("Couldn't load builder file: %s", error->message);
 		g_error_free (error);
+		return;
 	}
-	anjuta_preferences_add_from_builder (prefs, bxml,
+	anjuta_preferences_add_from_builder (prefs, bxml, plugin->settings,
 									"Messages", _("Messages"),
 									 ANJUTA_PIXMAP_MESSAGES);
-	notify_id = anjuta_preferences_notify_add_string (prefs, MESSAGES_TABS_POS, 
-	                                                on_notify_message_pref, plugin->msgman, NULL);
-		
+	
+	g_signal_connect (plugin->settings, "changed::messages-tab-position",
+	                  G_CALLBACK (on_notify_message_pref), plugin->msgman);		
 	g_object_unref (bxml);
 }
 
 static void
 ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
-	anjuta_preferences_notify_remove(prefs, notify_id);
 	anjuta_preferences_remove_page(prefs, _("Messages"));
 }
 
