@@ -42,6 +42,7 @@
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
 #include <libanjuta/interfaces/ianjuta-editor-search.h>
 #include <libanjuta/interfaces/ianjuta-editor-hover.h>
+#include <libanjuta/interfaces/ianjuta-editor-glade-signal.h>
 #include <libanjuta/interfaces/ianjuta-provider.h>
 
 #include <gtksourceview/gtksourceview.h>
@@ -760,7 +761,6 @@ sourceview_instance_init(Sourceview* sv)
 					 G_CALLBACK(on_backspace),sv);
 
 	g_object_set (G_OBJECT (sv->priv->view), "has-tooltip", TRUE, NULL);
-	gtk_source_view_set_smart_home_end(GTK_SOURCE_VIEW(sv->priv->view), FALSE);
 
 	/* Apply Preferences */
 	sourceview_prefs_init(sv);
@@ -1147,9 +1147,17 @@ static void ieditor_insert(IAnjutaEditor *editor, IAnjutaIterable* icell,
 	GtkTextIter iter;
 	Sourceview* sv = ANJUTA_SOURCEVIEW(editor);
 	sourceview_cell_get_iter (cell, &iter);
+
+	/* Avoid processing text that is inserted programatically */
+	g_signal_handlers_block_by_func (sv->priv->document,
+	                                 on_insert_text,
+	                                 sv);
 	
 	gtk_text_buffer_insert(GTK_TEXT_BUFFER(sv->priv->document),
 						   &iter, text, length);
+	g_signal_handlers_unblock_by_func (sv->priv->document,
+	                                   on_insert_text,
+	                                   sv);
 }
 
 /* Append text to buffer */
@@ -1161,8 +1169,16 @@ static void ieditor_append(IAnjutaEditor *editor, const gchar* text,
 
 	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(sv->priv->document),
 									   &iter);
+
+	/* Avoid processing text that is inserted programatically */
+	g_signal_handlers_block_by_func (sv->priv->document,
+	                                 on_insert_text,
+	                                 sv);
 	gtk_text_buffer_insert(GTK_TEXT_BUFFER(sv->priv->document),
 						   &iter, text, length);
+	g_signal_handlers_unblock_by_func (sv->priv->document,
+	                                   on_insert_text,
+	                                   sv);
 }
 
 static void ieditor_erase(IAnjutaEditor* editor, IAnjutaIterable* istart_cell, 
@@ -2459,6 +2475,12 @@ ihover_iface_init(IAnjutaEditorHoverIface* iface)
 	iface->display = ihover_display;
 }
 
+static void
+iglade_iface_init(IAnjutaEditorGladeSignalIface* iface)
+{
+	/* only signals */
+}
+
 ANJUTA_TYPE_BEGIN(Sourceview, sourceview, GTK_TYPE_VBOX);
 ANJUTA_TYPE_ADD_INTERFACE(idocument, IANJUTA_TYPE_DOCUMENT);
 ANJUTA_TYPE_ADD_INTERFACE(ifile, IANJUTA_TYPE_FILE);
@@ -2474,4 +2496,5 @@ ANJUTA_TYPE_ADD_INTERFACE(iprint, IANJUTA_TYPE_PRINT);
 ANJUTA_TYPE_ADD_INTERFACE(ilanguage, IANJUTA_TYPE_EDITOR_LANGUAGE);
 ANJUTA_TYPE_ADD_INTERFACE(isearch, IANJUTA_TYPE_EDITOR_SEARCH);
 ANJUTA_TYPE_ADD_INTERFACE(ihover, IANJUTA_TYPE_EDITOR_HOVER);
+ANJUTA_TYPE_ADD_INTERFACE(iglade, IANJUTA_TYPE_EDITOR_GLADE_SIGNAL);
 ANJUTA_TYPE_END;
